// -----------------------------------------------
// An analysis program that shows how to read ProMC files from HepSim
// and perform an analysis of reconstructed objects from Delphes 3.X on the fly
// S.Chekanov (ANL) chekanov@anl.gov
/// -----------------------------------------------

#include <stdexcept>
#include <iostream>
#include <sstream>
#include <memory>
#include <deque>
#include <stdlib.h>
#include <signal.h>
#include <stdio.h>
#include <algorithm>
#include "TROOT.h"
#include "TApplication.h"
#include "TMath.h"
#include "TString.h"
#include "TFile.h"
#include "TObjArray.h"
#include "TStopwatch.h"
#include "TDatabasePDG.h"
#include "TParticlePDG.h"
#include "TLorentzVector.h"
#include "TH1D.h"

#include "modules/Delphes.h"
#include "classes/DelphesStream.h"
#include "classes/DelphesClasses.h"
#include "classes/DelphesFactory.h"
#include "classes/DelphesModule.h"
#include "classes/DelphesFormula.h"

#include "ExRootAnalysis/ExRootTreeWriter.h"
#include "ExRootAnalysis/ExRootTreeBranch.h"
#include "ExRootAnalysis/ExRootProgressBar.h"
#include "ExRootAnalysis/ExRootResult.h"
#include "ExRootAnalysis/ExRootFilter.h"
#include "ExRootAnalysis/ExRootClassifier.h"
#include "ExRootAnalysis/ExRootTask.h"
#include "ExRootAnalysis/ExRootConfReader.h"

#include "promc/ProMCBook.h"

using namespace std;
extern vector<string> getfiles(char * directory);
extern void ConvertInput(promc::ProMCEvent&, double, double, DelphesFactory*, TObjArray*, TObjArray*, TObjArray*, TStopwatch*, TStopwatch*);


//---------------------------------------------------------------------------

static bool interrupted = false;
void SignalHandler(int sig)
{
	interrupted = true;
}

//---------------------------------------------------------------------------

int main(int argc, char *argv[])
{
	char appName[] = "analysis";
	stringstream message;
	ProMCBook *inputFile = 0;
	TFile *outputFile = 0;
	TStopwatch readStopWatch, procStopWatch;
	ExRootTreeWriter *treeWriter = 0;
	ExRootConfReader *confReader = 0;
	Delphes *modularDelphes = 0;
	DelphesFactory *factory = 0;
	TObjArray *allParticleOutputArray = 0, *stableParticleOutputArray = 0, *partonOutputArray = 0;
	Long64_t eventCounter, numberOfEvents;
        double xcross=0;
        int ntot=0;
        double cross=0;

	if(argc < 4)
	{
		cout << " Usage: " << appName << " config_file" << " output_file" << " input_file(s)" << endl;
		cout << " config_file - configuration file in Tcl format," << endl;
		cout << " output_file - output file in ROOT format," << endl;
		cout << " input_file(s) - file with the list of input file(s) in the ProMC format." << endl;
		return 1;
	}

	signal(SIGINT, SignalHandler);

	gROOT->SetBatch();

	int appargc = 1;
	char *appargv[] = {appName};
	TApplication app(appName, &appargc, appargv);

	vector <string> ntup=getfiles(argv[3]);     // ntuple list
	const int Nfiles = ntup.size();
	cout << " -> No of files to read:" << Nfiles << endl;


	try
	{

		outputFile = TFile::Open(argv[2], "CREATE");
		if(outputFile == NULL)
		{
			message << "can't open " << argv[2] << endl;
			throw runtime_error(message.str());
		}


		TH1D *debug = new TH1D("debug", "debug", 10, 0, 10);
		TH1D *jetPT = new TH1D("jetPT", "jet pt [GeV]", 200, 0, 1000);
                TH1D *jetM = new TH1D("jetMass", "jet mass [GeV]", 250, 0, 500);
		TH1D *muonPT = new TH1D("muonPT", "muonPT", 200, 0, 1000);
                TH1D *jet_tau1 = new TH1D("jetTau1", "jet Tau1", 100, 0, 800.0);
                TH1D *jet_tau2 = new TH1D("jetTau2", "jet Tau2", 100, 0, 800.0);
                TH1D *jet_tau3 = new TH1D("jetTau3", "jet Tau3", 100, 0, 800.0);
                TH1D *jet_tau32 = new TH1D("jetTau32", "jet Tau32", 100, 0, 2.0);
                TH1D *jet_tau21 = new TH1D("jetTau21", "jet Tau21", 100, 0, 2.0);


		treeWriter = new ExRootTreeWriter(outputFile, "Delphes");
		confReader = new ExRootConfReader;
		confReader->ReadFile(argv[1]);

		modularDelphes = new Delphes("Delphes");
		modularDelphes->SetConfReader(confReader);
		factory = modularDelphes->GetFactory();
		allParticleOutputArray = modularDelphes->ExportArray("allParticles");
		stableParticleOutputArray = modularDelphes->ExportArray("stableParticles");
		partonOutputArray = modularDelphes->ExportArray("partons");

		modularDelphes->InitTask();


		// start running over all files
		for (int i = 0; i < Nfiles && !interrupted; i++) {

			float donefiles= (float)i / (float)Nfiles;
			cout << "\nPercentage done = " << (int)(donefiles*100) <<  " %" << endl;
			cout << "\nfile to read:" << ntup[i] << endl;

			inputFile = new ProMCBook(ntup[i].c_str(), "r");
                        // get the version number
                        // int  h=inputFile->getVersion();
                        //cout << "Version = " << h << endl;
                        // get the description of this file
                        // string d=inputFile->getDescription();
                        // cout << "Description = " << d << endl;
                        // int  nev=inputFile->getEvents();
                        // cout << "Events = " << nev  << endl;
                        // get the header file with units, cross sections etc.
                        ProMCHeader header = inputFile->getHeader();
                       // cout << "Cross section=" << header.cross_section() << endl;
                       // Ok, here is the units
                        double kEV=(double)(header.momentumunit());
                       //cout << "Momentum units =" << kEV << endl;
                        double kLe=(double)(header.lengthunit());
                       // cout << "Length unit=" << kLe << endl;


			if(inputFile == NULL)
			{
				message << "can't open " << argv[i] << endl;
				throw runtime_error(message.str());
			}

			numberOfEvents = inputFile->getEvents();

			if(numberOfEvents <= 0) continue;

			ExRootProgressBar progressBar(numberOfEvents - 1);

			// Loop over all objects
			modularDelphes->Clear();
			treeWriter->Clear();
			readStopWatch.Start();
			for(eventCounter = 0; eventCounter < numberOfEvents && !interrupted; ++eventCounter)
			{
				if(inputFile->next() != 0) continue;
				ProMCEvent event = inputFile->get();

				readStopWatch.Stop();

				procStopWatch.Start();
				ConvertInput(event, kEV, kLe, factory,
				             allParticleOutputArray, stableParticleOutputArray, partonOutputArray,
				             &readStopWatch, &procStopWatch);

				modularDelphes->ProcessTask();
				procStopWatch.Stop();
                                ntot++;


                                // Do your analysis here -------------------------------------
                                // can create functions inside src/ directory
				// Check possible arrays in
				// https://cp3.irmp.ucl.ac.be/projects/delphes/wiki/WorkBook/Arrays
                                //
                                // Delphes/stableParticles 	Final state particles (Status==1) 
                                // PhotonIsolation/photons 	Isolated photons 
                                // MissingET/momentum 	Missing transverse energy 
                                // ScalarHT/energy 	Scalar sum of transverse momenta and energy of all reconstructed objects 
                                // ElectronIsolation/electrons 	Isolated electrons 
 

				// fill jets
				// const TObjArray *arrayJets = modularDelphes->ImportArray("FastJetFinder/jets");
                                // fill calibrated jets
                                const TObjArray *arrayJets = modularDelphes->ImportArray("JetEnergyScale/jets");
				TIter iteratorJets(arrayJets);
				Candidate *candidate;
				iteratorJets.Reset();
				while((candidate = static_cast<Candidate*>(iteratorJets.Next())))
				{
					const TLorentzVector &momentum = candidate->Momentum;
					// cout << " jet pT=" << momentum.Perp() << endl;
					jetPT->Fill(momentum.Perp());
                                        double Tau1 = candidate->Tau[0];
                                        double Tau2 = candidate->Tau[1];
                                        double Tau3 = candidate->Tau[2];
                                        if (momentum.Perp()>3000 && abs(momentum.Eta())<1.5) {
                                                     jetM->Fill(momentum.M());
                                                     jet_tau1->Fill(Tau1);
                                                     jet_tau2->Fill(Tau2);
                                                     jet_tau3->Fill(Tau3);
                                                     // cout << Tau1 << " " << Tau2 << " " << Tau3 << endl;
                                                     double tau32=0;
                                                     if (Tau2>0) tau32=Tau3/Tau2;
                                                     jet_tau32->Fill(tau32);
                                                     double tau21=0;
                                                     if (Tau1>0) tau21=Tau2/Tau1;
                                                     jet_tau21->Fill(tau21);

                                        }
                                   }


				const TObjArray *arrayMuons = modularDelphes->ImportArray("MuonMomentumSmearing/muons");
				TIter iteratorMuons(arrayMuons);
				iteratorMuons.Reset();
				while((candidate = static_cast<Candidate*>(iteratorMuons.Next())))
				{
					const TLorentzVector &momentum = candidate->Momentum;
					//cout << " muon pT=" << momentum.Perp() << endl;
					muonPT->Fill(momentum.Perp());

				}


                                // count events
				debug->Fill(0);
				modularDelphes->Clear();
				treeWriter->Clear();
				readStopWatch.Start();
				progressBar.Update(eventCounter);

			}
			progressBar.Update(eventCounter, eventCounter, kTRUE);
			progressBar.Finish();

                         // get accumulated cross section
                         ProMCStat stat = inputFile->getStatistics();
                         cross=stat.cross_section_accumulated();

                          // count average cross section
                          xcross=xcross+cross;

                          double lumi=(double)((i+1)*ntot)/xcross;
                          if (cross>0) cout << "--> Int. lumi analysed =" << lumi << " pb-1" << endl;


			inputFile->close();
			delete inputFile;
		}



		modularDelphes->FinishTask();
		cout << "** Exiting..." << endl;
		cout << "\n\n-- Write output file=" << argv[2]  << endl;
		outputFile->Write();
		outputFile->Print();
		outputFile->Close();

		delete modularDelphes;
		delete confReader;
		delete treeWriter;
		delete outputFile;

                 xcross=xcross/(double)Nfiles; // average cross for all files
                 cout << "Average cross section for all files = " << xcross << " pb"<< endl;
                 double lumi=(double)ntot/xcross;
                 cout << "Lumi for all files  =" << lumi << " pb-1" << endl;



		return 0;
	}
	catch(runtime_error &e)
	{
		if(treeWriter) delete treeWriter;
		if(outputFile) delete outputFile;
		cerr << "** ERROR: " << e.what() << endl;
		return 1;
	}
}
