// -----------------------------------------------
// An analysis program that shows how to read ProMC files from HepSim
// and perform an analysis of reconstructed objects from Delphes 3.X on the fly
// S.Chekanov (ANL) chekanov@anl.gov
/// -----------------------------------------------

#include <stdexcept>
#include <iostream>
#include <sstream>
#include <memory>
#include <deque>
#include <stdlib.h>
#include <signal.h>
#include <stdio.h>
#include <algorithm>
#include "TROOT.h"
#include "TApplication.h"
#include "TMath.h"
#include "TString.h"
#include "TFile.h"
#include "TObjArray.h"
#include "TStopwatch.h"
#include "TDatabasePDG.h"
#include "TParticlePDG.h"
#include "TLorentzVector.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TProfile.h"

#include "modules/Delphes.h"
#include "classes/DelphesStream.h"
#include "classes/DelphesClasses.h"
#include "classes/DelphesFactory.h"
#include "classes/DelphesModule.h"
#include "classes/DelphesFormula.h"

#include "ExRootAnalysis/ExRootTreeWriter.h"
#include "ExRootAnalysis/ExRootTreeBranch.h"
#include "ExRootAnalysis/ExRootProgressBar.h"
#include "ExRootAnalysis/ExRootResult.h"
#include "ExRootAnalysis/ExRootFilter.h"
#include "ExRootAnalysis/ExRootClassifier.h"
#include "ExRootAnalysis/ExRootTask.h"
#include "ExRootAnalysis/ExRootConfReader.h"

#include "ProMC/ProMCBook.h"

using namespace std;
extern vector<string> getfiles(char * directory);
extern void ConvertInput(promc::ProMCEvent&, double, double, DelphesFactory*, TObjArray*, TObjArray*, TObjArray*, TStopwatch*, TStopwatch*);

bool reorder(const TLorentzVector &a, const TLorentzVector &b)
{
   return a.Pt() > b.Pt();
}


//---------------------------------------------------------------------------

static bool interrupted = false;
void SignalHandler(int sig)
{
	interrupted = true;
}

//---------------------------------------------------------------------------

int main(int argc, char *argv[])
{
	char appName[] = "analysis";
	stringstream message;
	ProMCBook *inputFile = 0;
	TFile *outputFile = 0;
	TStopwatch readStopWatch, procStopWatch;
	ExRootTreeWriter *treeWriter = 0;
	ExRootConfReader *confReader = 0;
	Delphes *modularDelphes = 0;
	DelphesFactory *factory = 0;
	TObjArray *allParticleOutputArray = 0, *stableParticleOutputArray = 0, *partonOutputArray = 0;
	Long64_t eventCounter, numberOfEvents;
        double xcross=0;
        int ntot=0;
        double cross=0;

	if(argc < 4)
	{
		cout << " Usage: " << appName << " config_file" << " output_file" << " input_file(s)" << endl;
		cout << " config_file - configuration file in Tcl format," << endl;
		cout << " output_file - output file in ROOT format," << endl;
		cout << " input_file(s) - file with the list of input file(s) in the ProMC format." << endl;
		return 1;
	}

	signal(SIGINT, SignalHandler);

	gROOT->SetBatch();

	int appargc = 1;
	char *appargv[] = {appName};
	TApplication app(appName, &appargc, appargv);

	vector <string> ntup=getfiles(argv[3]);     // ntuple list
	const int Nfiles = ntup.size();
	cout << " -> No of files to read:" << Nfiles << endl;


	try
	{

		outputFile = TFile::Open(argv[2], "CREATE");
		if(outputFile == NULL)
		{
			message << "can't open " << argv[2] << endl;
			throw runtime_error(message.str());
		}


		TH1D *debug = new TH1D("debug", "debug", 10, 0, 10);
		TH1D *jetPT = new TH1D("jetPT", "jet pt [GeV]", 200, 0, 1000);
                TH1D *jetM = new TH1D("jetMass", "jet mass [GeV]", 250, 0, 500);
		TH1D *muonPT = new TH1D("muonPT", "muonPT", 200, 0, 1000);

  // Book histograms
  TH1D *histJetPT = new TH1D("jet_pt", "jetP_{T}", 100, 0.0, 1000.0);
  TH1D *histJetPTmatched = new TH1D("jet_pt_matched", "jetP_{T} matched", 100, 0.0, 1000.0);

  TH1D *histMissPT = new TH1D("missET", "missET_{T}", 100, 0.0, 1000.0);

  TH1D *histJetPT700 = new TH1D("jet_pt700", "jetP_{T} 500", 50, 500.0, 1500.0);
  TH1D *histDijetMass = new TH1D("dijet_mass", "Dijet mass", 100, 500.0, 4000.0);
  TH1D *histDijetMassTrue = new TH1D("dijet_mass_true", "Dijet mass true", 100, 500.0, 4000.0);

  TH1D *histN = new TH1D("numberN", "Number", 100, 0.0, 100.0);
  TH1D *histMass = new TH1D("jet_mass", "mass(4GeV)", 100, 0.0, 400.0);
  TH1D *histMassTrue = new TH1D("jet_mass_true", "mass(4 GeV)true", 100, 0.0, 400.0);

  TH2D *histPTtruePTjet  = new TH2D("PTtruePTjet","PTtruePTjet",100,50,1200,100,50,1200); histPTtruePTjet->Sumw2();
  TH2D *histMtrueMjet  = new TH2D("MtrueMjet","PTtruePTjet",80,50,400,80,50,400); histMtrueMjet->Sumw2();
  TH2D *histPTMjet  = new TH2D("PTvsMjet","PTvsMjet",40,200,3000,80,50,400); histPTMjet->Sumw2();

   TH1D *histMass0 = new TH1D("jet_mass0", "mass pt500", 100, 0.0, 400.0);
   TH1D *histMass1 = new TH1D("jet_mass1", "mass pt800", 100, 0.0, 400.0);
   TH1D *histMass2 = new TH1D("jet_mass2", "mass pt1200", 100, 0.0, 400.0);
   TH1D *histMass3 = new TH1D("jet_mass3", "mass pt1600", 100, 0.0, 400.0);
   TH1D *histMass4 = new TH1D("jet_mass4", "mass pt2000", 100, 0.0, 400.0);
   TH1D *histMass5 = new TH1D("jet_mass5", "mass pt2400", 100, 0.0, 400.0);
   TH1D *histMass6 = new TH1D("jet_mass6", "mass pt2800", 100, 0.0, 400.0);

   int hb=500;
  double xmin=-2;
  double xmax=3;
  TH1D *histRes1 = new TH1D("histRes50", "histRes50", hb, xmin,xmax);
  TH1D *histRes2 = new TH1D("histRes100", "histRes100", hb, xmin,xmax);
  TH1D *histRes3 = new TH1D("histRes200", "histRes200", hb, xmin,xmax);
  TH1D *histRes4 = new TH1D("histRes300", "histRes300", hb, xmin,xmax);
  TH1D *histRes5 = new TH1D("histRes400", "histRes400", hb, xmin,xmax);
  TH1D *histRes6 = new TH1D("histRes500", "histRes500", hb, xmin,xmax);
  TH1D *histRes7 = new TH1D("histRes600", "histRes600", hb, xmin,xmax);
  TH1D *histRes8 = new TH1D("histRes700", "histRes700", hb, xmin,xmax);
  TH1D *histRes9 = new TH1D("histRes800", "histRes800", hb, xmin,xmax);
  TH1D *histRes10 = new TH1D("histRes900", "histRes900", hb, xmin,xmax);
  TH1D *histRes11 = new TH1D("histRes1000", "histRes1000", hb, xmin,xmax);

  hb=400;
  xmin=0;
  xmax=2000;
  TH1D *histAv1 = new TH1D("histAv50", "histAv50", hb, xmin,xmax);
  TH1D *histAv2 = new TH1D("histAv100", "histAv100", hb, xmin,xmax);
  TH1D *histAv3 = new TH1D("histAv200", "histAv200", hb, xmin,xmax);
  TH1D *histAv4 = new TH1D("histAv300", "histAv300", hb, xmin,xmax);
  TH1D *histAv5 = new TH1D("histAv400", "histAv400", hb, xmin,xmax);
  TH1D *histAv6 = new TH1D("histAv500", "histAv500", hb, xmin,xmax);
  TH1D *histAv7 = new TH1D("histAv600", "histAv600", hb, xmin,xmax);
  TH1D *histAv8 = new TH1D("histAv700", "histAv700", hb, xmin,xmax);
  TH1D *histAv9 = new TH1D("histAv800", "histAv800", hb, xmin,xmax);
  TH1D *histAv10 = new TH1D("histAv900", "histAv900", hb, xmin,xmax);
  TH1D *histAv11 = new TH1D("histAv1000", "histAv1000", hb, xmin,xmax);



		treeWriter = new ExRootTreeWriter(outputFile, "Delphes");
		confReader = new ExRootConfReader;
		confReader->ReadFile(argv[1]);

		modularDelphes = new Delphes("Delphes");
		modularDelphes->SetConfReader(confReader);
		factory = modularDelphes->GetFactory();
		allParticleOutputArray = modularDelphes->ExportArray("allParticles");
		stableParticleOutputArray = modularDelphes->ExportArray("stableParticles");
		partonOutputArray = modularDelphes->ExportArray("partons");

		modularDelphes->InitTask();


		// start running over all files
		for (int i = 0; i < Nfiles && !interrupted; i++) {

			float donefiles= (float)i / (float)Nfiles;
			cout << "\nPercentage done = " << (int)(donefiles*100) <<  " %" << endl;
			cout << "\nfile to read:" << ntup[i] << endl;

			inputFile = new ProMCBook(ntup[i].c_str(), "r");
                        // get the version number
                        // int  h=inputFile->getVersion();
                        //cout << "Version = " << h << endl;
                        // get the description of this file
                        // string d=inputFile->getDescription();
                        // cout << "Description = " << d << endl;
                        // int  nev=inputFile->getEvents();
                        // cout << "Events = " << nev  << endl;
                        // get the header file with units, cross sections etc.
                        ProMCHeader header = inputFile->getHeader();
                       // cout << "Cross section=" << header.cross_section() << endl;
                       // Ok, here is the units
                        double kEV=(double)(header.momentumunit());
                       //cout << "Momentum units =" << kEV << endl;
                        double kLe=(double)(header.lengthunit());
                       // cout << "Length unit=" << kLe << endl;


			if(inputFile == NULL)
			{
				message << "can't open " << argv[i] << endl;
				throw runtime_error(message.str());
			}

			numberOfEvents = inputFile->getEvents();

			if(numberOfEvents <= 0) continue;

			ExRootProgressBar progressBar(numberOfEvents - 1);

			// Loop over all objects
			modularDelphes->Clear();
			treeWriter->Clear();
			readStopWatch.Start();
			for(eventCounter = 0; eventCounter < numberOfEvents && !interrupted; ++eventCounter)
			{
				if(inputFile->next() != 0) continue;
				ProMCEvent event = inputFile->get();

				readStopWatch.Stop();

				procStopWatch.Start();
				ConvertInput(event, kEV, kLe, factory,
				             allParticleOutputArray, stableParticleOutputArray, partonOutputArray,
				             &readStopWatch, &procStopWatch);

				modularDelphes->ProcessTask();
				procStopWatch.Stop();
                                ntot++;


                                // Do your analysis here -------------------------------------
                                // can create functions inside src/ directory
				// Check possible arrays in
				// https://cp3.irmp.ucl.ac.be/projects/delphes/wiki/WorkBook/Arrays
                                //
                                // Delphes/stableParticles 	Final state particles (Status==1) 
                                // PhotonIsolation/photons 	Isolated photons 
                                // MissingET/momentum 	Missing transverse energy 
                                // ScalarHT/energy 	Scalar sum of transverse momenta and energy of all reconstructed objects 
                                // ElectronIsolation/electrons 	Isolated electrons 
 

                                vector<TLorentzVector> reco_jets;
                                vector<TLorentzVector> true_jets;
				// -------------  fill reco jets
				// const TObjArray *arrayJets = modularDelphes->ImportArray("FastJetFinder/jets");
                                const TObjArray *arrayJets = modularDelphes->ImportArray("JetEnergyScale/jets");
				TIter iteratorJets(arrayJets);
				Candidate *candidate;
				iteratorJets.Reset();
				while((candidate = static_cast<Candidate*>(iteratorJets.Next())))
				{
					const TLorentzVector &momentum = candidate->Momentum;
					// cout << " jet pT=" << momentum.Perp() << endl;
					jetPT->Fill(momentum.Perp());
                                        if (momentum.Perp()>20 && abs(momentum.Eta())<1.6) reco_jets.push_back(momentum);
                                        if (momentum.Perp()>2000 && abs(momentum.Eta())<1.0) jetM->Fill(momentum.M());

                                       

				}


                                // -------------------  fill true jets
                                const TObjArray *arrayTrueJets = modularDelphes->ImportArray("GenJetFinder/jets");
                                TIter iteratorTrueJets(arrayTrueJets);
                                Candidate *truejet;
                                iteratorTrueJets.Reset();
                                while((truejet = static_cast<Candidate*>(iteratorTrueJets.Next())))
                                {
                                        const TLorentzVector &momentum = truejet->Momentum;
                                        // cout << " jet pT=" << momentum.Perp() << endl;
                                        if (momentum.Perp()>20 && abs(momentum.Eta())<1.6) true_jets.push_back(momentum); 
                                }

                    
                                 std::sort(reco_jets.begin(), reco_jets.end(), reorder);
                                 std::sort(true_jets.begin(), true_jets.end(), reorder);
 
                                

                                // resolution plot
                                for(int i = 0; i<reco_jets.size(); i++){
                                  TLorentzVector rjet = (TLorentzVector)reco_jets.at(i);
                                  double pt= rjet.Perp();
                                  double e=  rjet.E();
                                  double eta = rjet.Eta();
                                  double phi = rjet.Phi();
                                  double mass = rjet.M();
                                  // cout << i << " " << pt << endl;

                                  histPTMjet->Fill(pt,mass);
                                  if (pt>600 && pt<1000) histMass0->Fill(mass);
                                  if (pt>800 && pt<1200) histMass1->Fill(mass);
                                  if (pt>1200 && pt<1600) histMass2->Fill(mass);
                                  if (pt>1600 && pt<2000) histMass3->Fill(mass);
                                  if (pt>2000 && pt<2400) histMass4->Fill(mass);
                                  if (pt>2400 && pt<2800) histMass5->Fill(mass);
                                  if (pt>2800 && pt<3200) histMass6->Fill(mass);


                                  for(int j = 0; j<true_jets.size(); j++){
                                       TLorentzVector gjet = (TLorentzVector)true_jets.at(j);
                                       double ptT= gjet.Perp();
                                       double eT=  gjet.E();
                                       double etaT = gjet.Eta();
                                       double phiT = gjet.Phi();
                                       double Deta=fabs(eta-etaT);
                                       double Dphi=fabs(phi-phiT);
                                       double R=sqrt(Deta*Deta+Dphi*Dphi);
                                       if (R<0.1)  {
                                         // TLorentzVector l;
                                         // l.SetPtEtaPhiM(pt,eta,phi,mass);
                                         // goodjets_matched.push_back(l);
                                         histPTtruePTjet->Fill(ptT,pt);
                                         histMtrueMjet->Fill(gjet.M(),mass);
                                         double r=( (pt-ptT)/ ptT);
                                         double rr=pt/ptT;

                if (ptT>50 &&  ptT < 100)   {histRes1->Fill(r); histAv1->Fill(ptT);}
                if (ptT>100 && ptT < 150)   {histRes2->Fill(r); histAv2->Fill(ptT);}
                if (ptT>200 && ptT < 300)   {histRes3->Fill(r); histAv3->Fill(ptT);}
                if (ptT>300 && ptT < 400)   {histRes4->Fill(r); histAv4->Fill(ptT);}
                if (ptT>400 && ptT < 500)   {histRes5->Fill(r); histAv5->Fill(ptT);}
                if (ptT>500 && ptT < 600)   {histRes6->Fill(r); histAv6->Fill(ptT);}
                if (ptT>600 && ptT < 700)   {histRes7->Fill(r); histAv7->Fill(ptT);}
                if (ptT>700 && ptT < 800)   {histRes8->Fill(r); histAv8->Fill(ptT);}
                if (ptT>800 && ptT < 900)   {histRes9->Fill(r); histAv9->Fill(ptT);}
                if (ptT>900 && ptT <  1000)  {histRes10->Fill(r); histAv10->Fill(ptT);}
                if (ptT>1000 && ptT < 1200)  {histRes11->Fill(r); histAv11->Fill(ptT);}
 
                }

               } // end of true jet
              } // end of reco jet


       // fill dijet mass
      if (reco_jets.size()>1){
         TLorentzVector j1=reco_jets[0];
         TLorentzVector j2=reco_jets[1];
         TLorentzVector jj=j1+j2;
         double jm=jj.M();
         histDijetMass->Fill(jm);
      }

                               

				const TObjArray *arrayMuons = modularDelphes->ImportArray("MuonMomentumSmearing/muons");
				TIter iteratorMuons(arrayMuons);
				iteratorMuons.Reset();
				while((candidate = static_cast<Candidate*>(iteratorMuons.Next())))
				{
					const TLorentzVector &momentum = candidate->Momentum;
					//cout << " muon pT=" << momentum.Perp() << endl;
					muonPT->Fill(momentum.Perp());

				}


                                // count events
				debug->Fill(0);
				modularDelphes->Clear();
				treeWriter->Clear();
				readStopWatch.Start();
				progressBar.Update(eventCounter);

			}
			progressBar.Update(eventCounter, eventCounter, kTRUE);
			progressBar.Finish();

                         // get accumulated cross section
                         ProMCStat stat = inputFile->getStatistics();
                         cross=stat.cross_section_accumulated();

                          // count average cross section
                          xcross=xcross+cross;

                          double lumi=(double)((i+1)*ntot)/xcross;
                          if (cross>0) cout << "--> Approx lumi so far=" << lumi << " pb-1" << endl;


			inputFile->close();
			delete inputFile;
		}



		modularDelphes->FinishTask();
		cout << "** Exiting..." << endl;
		cout << "\n\n-- Write output file=" << argv[2]  << endl;
		outputFile->Write();
		outputFile->Print();
		outputFile->Close();

		delete modularDelphes;
		delete confReader;
		delete treeWriter;
		delete outputFile;

                 xcross=xcross/(double)Nfiles; // average cross for all files
                 cout << "Average cross section for all files = " << xcross << " pb"<< endl;
                 double lumi=(double)ntot/xcross;
                 cout << "Lumi for all files  =" << lumi << " pb-1" << endl;



		return 0;
	}
	catch(runtime_error &e)
	{
		if(treeWriter) delete treeWriter;
		if(outputFile) delete outputFile;
		cerr << "** ERROR: " << e.what() << endl;
		return 1;
	}
}
